/*
 * This file is part of the KFTPGrabber project
 *
 * Copyright (C) 2003-2004 by the KFTPGrabber developers
 * Copyright (C) 2003-2004 Jernej Kos <kostko@jweb-network.net>
 * Copyright (C) 2004 Markus Brueffer <markus@brueffer.de>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
 * NON-INFRINGEMENT.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */

#ifndef MAINWINDOW_H_
#define MAINWINDOW_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <tqtimer.h>

#include <tdemainwindow.h>
#include <tdeaction.h>

class KFTPZeroConfListView;
class MainActions;

namespace KFTPWidgets {
namespace Bookmarks {
  class Sidebar;
}

  class Sidebar;
  class TrafficGraph;
  class QueueView;
}

/**
 * @short Application Main Window
 * @author Jernej Kos <kostko@jweb-network.net>
 */
class MainWindow : public TDEMainWindow
{
TQ_OBJECT
public:
    /**
     * Class constructor.
     */
    MainWindow();
    
    /**
     * Class destructor.
     */
    ~MainWindow();
    
    /**
     * Get main actions instance.
     *
     * @return A valid instance of MainActions
     */
    MainActions *getActions() { return m_actions; }
protected:
    /**
     * Initialize the traffic graph widget.
     */
    void initTrafficGraph();
    
    /**
     * Initialize the status bar.
     */
    void initStatusBar();
    
    /**
     * Create the main user interface.
     */
    void initMainView();
        
    /**
     * Initialize sidebars.
     */
    void initSidebars();
    
    /**
     * This method gets called when the user attempts to close the
     * application.
     */
    bool queryClose();
private:
    TQTimer *m_graphTimer;
    MainActions *m_actions;

    TDEActionMenu *m_bookmarkMenu;
    TDEActionMenu *m_zeroconfMenu;
    TDEActionMenu *m_walletMenu;

    KFTPWidgets::Sidebar *m_leftSidebar;
    KFTPWidgets::Sidebar *m_bottomSidebar;
    
    KFTPWidgets::Bookmarks::Sidebar *m_bookmarkSidebar;
    
    KFTPWidgets::TrafficGraph *m_trafficGraph;
    
    KFTPZeroConfListView *m_zeroconfSidebar;
    KFTPWidgets::QueueView *m_queueView;
public slots:
    void initBookmarkMenu();
    void slotQuickConnect(const TQString &title = TQString::null, const TQString &host = TQString::null, int port = 21);
private slots:
    void appShutdown();
    void showBookmarkEditor();
    void slotUpdateStatusBar();
    void slotReadBookmarks();
    void slotUpdateTrafficGraph();
    void slotConfigChanged();
};

#endif
