/***************************************************************************
 *   Copyright (C) 2003 by Jernej Kos                                      *
 *   kostko@jweb-network.net                                               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#ifndef FOOBAR_H
#define FOOBAR_H

/**
@author Jernej Kos
*/
class FooBar{
public:


    ~FooBar();

};

#endif
