/*
 * This file is part of the KFTPGrabber project
 *
 * Copyright (C) 2003-2005 by the KFTPGrabber developers
 * Copyright (C) 2003-2005 Jernej Kos <kostko@jweb-network.net>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * is provided AS IS, WITHOUT ANY WARRANTY; without even the implied
 * warranty of MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, and
 * NON-INFRINGEMENT.  See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Steet, Fifth Floor, Boston,
 * MA 02110-1301, USA.
 *
 * In addition, as a special exception, the copyright holders give
 * permission to link the code of portions of this program with the
 * OpenSSL library under certain conditions as described in each
 * individual source file, and distribute linked combinations
 * including the two.
 *
 * You must obey the GNU General Public License in all respects
 * for all of the code used other than OpenSSL.  If you modify
 * file(s) with this exception, you may extend this exception to your
 * version of the file(s), but you are not obligated to do so.  If you
 * do not wish to do so, delete this exception statement from your
 * version.  If you delete this exception statement from all source
 * files in the program, then also delete it here.
 */

#ifndef KFTPWIDGETSBALLOON_H
#define KFTPWIDGETSBALLOON_H

#include <tqwidget.h>

namespace KFTPWidgets {

/**
 * A little balloon for notifications taken from Kopete with some modifications
 * for KFTPGrabber.
 *
 * @author Malte Starostik <malte@kde.org>
 * @author Duncan Mac-Vicar Prett <duncan@kde.org>
 */
class Balloon : public TQWidget
{
TQ_OBJECT
public:
    /**
     * Class constructor.
     *
     * @param text The text to display inside the balloon
     * @param pic Name of the pixmap to display inside the balloon
     */
    Balloon(const TQString &text, const TQString &pic);
    
    /**
     * Set widget anchor.
     *
     * @param anchor Anchor coordinates
     */
    void setAnchor(const TQPoint &anchor);
    
    /**
     * Intercept mouse press events.
     */
    void mousePressEvent(TQMouseEvent *e);
signals:
    void signalButtonClicked();
    void signalIgnoreButtonClicked();
    void signalBalloonClicked();
protected:
    void updateMask();
private:
    TQPoint m_anchor;
};

}

#endif
