<!DOCTYPE kpartgui SYSTEM "kpartgui.dtd">
<kpartgui name="kftpgrabber" version="1">

<MenuBar>
  <text>Main Menu</text>
  
  <Menu name="file">
    <text>&amp;File</text>
    
    <Action name="file_newsession" />
    <Action name="file_quick_connect" />
  </Menu>

  <Action name="bookmarks" />
  
</MenuBar>

<ToolBar name="mainToolBar" noMerge="1" fullWidth="true"><text>Main Toolbar</text>
  <Action name="file_newsession" />
  <Action name="file_quick_connect" />
  <Action name="transfermode" />
</ToolBar>

<ToolBar name="bookmarkToolBar" noMerge="1" fullWidth="true"><text>Bookmark Toolbar</text>
  <Action name="bookmark_new" />
  <Action name="bookmark_edit2" />
  <Action name="bookmark_delete" />
  <Separator/>
  <Action name="bookmark_subcat" />
</ToolBar>
</kpartgui>
